@@ -1,9 +1,22 @@
 
 check.inputs.mlam <- function (ncpu, availmemGb, colname.trait, map, pheno,
-                  geno, Zmat )
+                  geno, Zmat, gamma, falseposrate=NULL )
 {
   ## internal function for AM
 
+if(!is.null(falseposrate)){
+  if(!is.numeric(falseposrate)){
+    message("Error: the falseposrate parameter must be a value between 0 and 1. \n")
+    return(TRUE)
+  } else {
+   if(falseposrate < 0 | falseposrate > 1){
+    message("Error: the falseposrate parameter must be a value between 0 and 1. \n")
+    return(TRUE)
+   }
+ }
+}
+
+
 
 if(is.null(colname.trait)){
    message("Error: the name of the column containing the trait data must be given. \n")
@@ -170,6 +183,33 @@ if(is.null(map)){
  }
 
 
+## check gamma is between 0 and 1 if specified. 
+if(!is.null(gamma)){
+  if(!is.numeric(gamma))
+  { 
+    message("Error: a non-numeric value has been set for the gamma parameter. ")
+    message("       Valid values are between 0 and 1. ")
+    return(TRUE)
+  } else {
+
+     if(gamma < 0)
+     {
+        message("Error: the gamma parameter cannot be negative. ")
+        message("       Valid values are between 0 and 1. ")
+        return(TRUE)
+     } 
+     if(gamma > 1 )
+     {
+        message("Error: the gamma parameter cannot be greater than 1. ")
+        message("       Valid values are between 0 and 1. ")
+        return(TRUE)
+    }
+
+ }  ## end if else
+
+} ## end outer if
+
+
 
   return(FALSE)
 
